from rest_framework_simplejwt.views import TokenRefreshView, TokenVerifyView
from rest_framework_simplejwt.authentication import JWTAuthentication
from rest_framework_simplejwt.tokens import RefreshToken, TokenError, AccessToken



def multiple(field, arr):
    if len(arr) == 0:
        return field + ' in (null)'
    cond = ''
    for el in arr:
        cond += str(el) + ','
    return field + ' in (' + cond[:-1] + ')'


def single(field, val, prepend=''):
    if val is None:
        return ''
    return prepend + field + ' = ' + str(val)


def profile_group(p, exclude_holding=True, prepend=''):
    if p is None:
        return prepend + "h.value != 0" if exclude_holding else ''
    if p.hierarchy.value == 0 and exclude_holding:
        return prepend + "false"
    return prepend + "pg.id = " + str(p.id)


def uc_status(uc, prepend=''):
    if uc == '' or uc is None:
        return ""
    if uc == 'Y':
        return prepend + "uc_status = 'Y'"
    return prepend + "uc_status = 'N' or uc_status is null"


def is_target(it, prepend=''):
    if it is None:
        return ""
    if it:
        return prepend + "ctg.id is not null"
    return prepend + "ctg.id is null"


def fuzzy(string, field, limit, with_condition=True, prepend='', append=''):
    if string == "":
        return ""
    if limit is None:
        filter_text = f"levenshtein(lower({field}), '{string.lower()}')"
    else:
        filter_text = f"levenshtein_less_equal(lower({field}), '{string.lower()}', 1, 0, 1, {limit})"
    if with_condition and limit is not None:
        filter_text += f" <= {limit}"
    return prepend + filter_text + append



import datetime
import math
from .general import safe_division


def paginate(resp, pagination, total):
    return {
        'total_pages': math.ceil(total / pagination['size']),
        'results': resp
    }


def populate_by_month(query_response, first_date, date_index=0, default=0, keys=["rev"]):
    resp = []
    it = first_date
    current_month = first_date.month
    aux_it = 0
    n_keys = len(keys)
    while it.month == current_month:
        if aux_it < len(query_response) and it == query_response[aux_it][date_index]:
            r = {'date': it}
            for i in range(n_keys):
                r[keys[i]] = query_response[aux_it][1 + i]
            resp.append(r)
            aux_it += 1
        else:
            r = {'date': it}
            for i in range(n_keys):
                r[keys[i]] = default
            resp.append(r)
        it = it + datetime.timedelta(days=1)
    return resp


def populate_by_aggregation(query_response, first_date, final_date, agg, date_index=0, default=0, keys=["rev"]):
    resp = []
    it = first_date
    aux_it = 0
    n_keys = len(keys)
    while 1:
        if aux_it < len(query_response) and agg.datetime_to_string(it) == query_response[aux_it][date_index]:
            r = {'date': it}
            for i in range(n_keys):
                r[keys[i]] = query_response[aux_it][1 + i]
            resp.append(r)
            aux_it += 1
        else:
            r = {'date': it}
            for i in range(n_keys):
                r[keys[i]] = default
            resp.append(r)
        it = agg.get_next(it)
        if it > final_date:
            return resp


def obj_list(query, fields):
    n = len(fields)
    resp = []
    for line in query:
        r = {}
        for i in range(n):
            r[fields[i]] = line[i]
        resp.append(r)
    return resp


def obj(query, fields):
    r = {}
    for i in range(len(fields)):
        r[fields[i]] = query[i]
    return r


def obj_from_lines(query, index=1):
    r = {}
    for line in query:
        r[line[0]] = line[index]
    return r


def obj_from_lines_division(query):
    r = {}
    for line in query:
        r[line[0]] = safe_division(line[1], line[2])
    return r


def sum_index(query, index):
    s = 0
    for line in query:
        s += line[index]
    return s


def populate_array_by_dates(query, initial_date, final_date):
    i = 0
    resp = []
    it = initial_date
    while it <= final_date:
        percentage = 0
        if i < len(query) and query[i][0] == it:
            percentage = query[i][1]
            i += 1
        resp.append(percentage)
        it = it + datetime.timedelta(days=1)
    return resp


def populate_and_group_by_dates(query, initial_date, final_date, date_index=0, group_index=1, value_index=2):
    n = len(query)
    resp = []
    it = initial_date
    aux = 0
    while 1:
        aux_obj = {'date': it}
        while aux < n and it == query[aux][date_index]:
            aux_obj[query[aux][group_index]] = query[aux][value_index]
            aux += 1
        resp.append(aux_obj)
        if it == final_date:
            break
        it = it + datetime.timedelta(days=1)
    return resp


def group_by(query, field):
    resp = []
    n = len(query)
    it = 0
    while it < n:
        aux_obj = {field: query[it][0]}
        while it < n and query[it][0] == aux_obj[field]:
            aux_obj[query[it][1]] = query[it][2]
            it += 1
        resp.append(aux_obj)
    return resp



import copy
import datetime
import json
import os
from django.http import Http404
from .exceptions import ValidationError


def get_infra_env():
    if os.environ.get("ENVIRON") == 'production':
        return 'production'
    return 'staging'


def get_base_lambda_name():
    return "arccanet-infra-{}-".format(get_infra_env())


def parse_json(data):
    try:
        return json.loads(data)
    except Exception:
        raise ValidationError("Invalid JSON: {}".format(data))


def get_store_today(timezone):
    return (datetime.datetime.utcnow() - datetime.timedelta(minutes=timezone)).date()


def to_none(val):
    if val == '':
        return None
    return val


def search_object(model, pk):
    q_set = model.objects.filter(pk=pk)
    if not q_set.exists():
        raise Http404
    return q_set[0]


def search_object_with_soft_delete(model, pk):
    q_set = model.objects.filter(pk=pk, deleted_at=None)
    if not q_set.exists():
        raise Http404
    return q_set[0]


def search_object_custom(model, obj):
    q_set = model.objects.filter(**obj)
    if not q_set.exists():
        return None
    return q_set[0]


def search_model_by_fields(data, fields, model, default_obj={}):
    for field in fields:
        if data.get(field) is not None:
            obj = copy.copy(default_obj)
            obj[field] = data[field]
            customer = search_object_custom(model, obj)
            if customer is not None:
                return customer, field
    return None, None


def get_starting_minutes(as_time=False):
    starting_minutes = 360
    if as_time:
        return datetime.time(starting_minutes // 60, starting_minutes % 60)
    return starting_minutes


def safe_division(a, b, default=None):
    return a / b if b != 0 else default


class AggregationPeriod:
    def __init__(self, agg):
        if agg == "day":
            def get_next(it):
                return it + datetime.timedelta(days=1)

            self.format = "YYYY-MM-DD"
            self.datetime_to_string = str
            self.get_next = get_next
            self.approximate_days = 1
        elif agg == "week":
            def datetime_to_string(week):
                iso = week.isocalendar()
                return str(iso[0]) + "-" + ("0" if iso[1] < 10 else "") + str(iso[1])

            def get_next(it):
                iso = it.isocalendar()
                return datetime.date.fromisocalendar(iso[0], iso[1], 7) + datetime.timedelta(days=1)

            self.format = "IYYY-IW"
            self.datetime_to_string = datetime_to_string
            self.get_next = get_next
            self.approximate_days = 7
        elif agg == "month":
            def datetime_to_string(month):
                return month.strftime('%Y-%m')

            def get_next(it):
                if it.month != 12:
                    return datetime.date(it.year, it.month + 1, 1)
                return datetime.date(it.year + 1, 1, 1)

            self.format = "YYYY-MM"
            self.datetime_to_string = datetime_to_string
            self.get_next = get_next
            self.approximate_days = 30
        elif agg == "year":
            def datetime_to_string(year):
                return year.strftime('%Y')

            def get_next(it):
                return datetime.date(it.year + 1, 1, 1)

            self.format = "YYYY"
            self.datetime_to_string = datetime_to_string
            self.get_next = get_next
            self.approximate_days = 365



from rest_framework.views import APIView as View
from django.db import connection


class APIView(View):
    cursor = None

    def dispatch(self, request, *args, **kwargs):
        self.cursor = connection.cursor()
        resp = super(APIView, self).dispatch(request, *args, **kwargs)
        self.cursor.close()
        return resp



from setuptools import setup

setup(
    name='arccanet',
    version='0.0.3',
    author='Arcca Tech Team',
    author_email='rodrigo.almeida@arcca.io',
    packages=['arccanet'],
    description='Base package for building arccanet backend',
    install_requires=[
        "Django==3.2.5",
        "djangorestframework==3.12.4",
        "djangorestframework-simplejwt==4.7.2",
    ],
)
